player = {}
obstacles = {}
rockets = {}
last_rocket_spawn = 0
rocket_cooldown = 0.5
start_button = {}
hud = {}
background = {}

input_dir = {x = 0, y = 0}
global_scale_factor = 4
window_width, window_height = love.graphics.getDimensions()
collision_rects = {
}
last_pos = {x = 0, y = 0}
game_paused = 1

function background_init()
	background.sprite = love.graphics.newImage("sprites/background.png")
	background.sprite:setFilter("nearest")
end

function start_button_init()
	start_button.sprite = love.graphics.newImage("sprites/start.png")
	start_button.sprite:setFilter("nearest")
	start_button.pixel_width, start_button.pixel_height = start_button.sprite:getPixelDimensions()
end

function player_init()
	player.sprite = love.graphics.newImage("sprites/ufo_02.png")
	player.sprite:setFilter("nearest")
	player.scale_factor = global_scale_factor
	player.pixel_width, player.pixel_ignore = player.sprite:getPixelDimensions()
	player.pixel_height = player.pixel_width
	player.pos_x = 0
	player.pos_y = 0
	player.rect_x = player.pos_x + (player.pixel_width * player.scale_factor)
	player.rect_y = player.pos_y + (player.pixel_height * player.scale_factor)
	player.rotation = 0
	player.speed = 400
	player.velocity_x = 0
	player.velocity_y = 0
end

function obstacle_init(sprite, x, y)
	local obj = {}
	obj.sprite = love.graphics.newImage(sprite)
	obj.sprite:setFilter("nearest")
	obj.scale_factor = global_scale_factor
	obj.pixel_width, obj.pixel_height = obj.sprite:getPixelDimensions()
	obj.pos_x = x
	obj.pos_y = y
	obj.rect_x = obj.pos_x + (obj.pixel_width * obj.scale_factor)
	obj.rect_y = obj.pos_y + (obj.pixel_height * obj.scale_factor)
	table.insert(obstacles, obj)
end

function rocket_init(dir)
	local rocket = {}
	rocket.sprite = love.graphics.newImage("sprites/rocket.png")
	rocket.sprite:setFilter("nearest")
	rocket.scale_factor_y = global_scale_factor
	rocket.pixel_width, rocket.pixel_height = rocket.sprite:getPixelDimensions()
	rocket.visible = 0
	if dir == 1 then
		rocket.dir = 1
		rocket.scale_factor_x = -global_scale_factor
		rocket.pos_x = 0 - (rocket.pixel_width * global_scale_factor)
	elseif dir == 2 then
		rocket.dir = -1
		rocket.scale_factor_x = global_scale_factor
		rocket.pos_x = window_width
	end
	rocket.pos_y = player.pos_y
	rocket.rotation = 0
	rocket.speed = 450
	table.insert(rockets, rocket)
end

function hud_init()
	hud.sprite = love.graphics.newImage("sprites/hud.png")
	hud.sprite:setFilter("nearest")
	hud.pixel_width, hud.pixel_height = hud.sprite:getPixelDimensions()
end

function love.load()
	background_init()
	start_button_init()
	player_init()
	hud_init()
	love.window.setMode(800, 600, {fullscreen = false, fullscreentype = "desktop"})
end

function love.keypressed(key, scancode, isrepeat)
	if scancode == "f8" then
		love.event.quit()
	end

	if scancode == "f11" then
		fullscreen = not fullscreen
		love.window.setFullscreen(fullscreen, "desktop")
	end

	if scancode == "left" or scancode == "a" then
		input_dir.x = input_dir.x - 1
	end
	if scancode == "right" or scancode == "d" then
		input_dir.x = input_dir.x + 1
	end
	
	if scancode == "up" or scancode == "w" then
		input_dir.y = input_dir.y - 1
	end
	if scancode == "down" or scancode == "s" then
		input_dir.y = input_dir.y + 1
	end	
end

function love.keyreleased(key, scancode, isrepeat)
	if scancode == "left" or scancode == "a" then
		input_dir.x = input_dir.x + 1
	end
	if scancode == "right" or scancode == "d" then
		input_dir.x = input_dir.x - 1
	end
	
	if scancode == "up" or scancode == "w" then
		input_dir.y = input_dir.y + 1
	end
	if scancode == "down" or scancode == "s" then
		input_dir.y = input_dir.y - 1
	end	
end

function love.mousepressed(x, y, button, istouch)
	if x > (window_width / 2) - ((start_button.pixel_width * global_scale_factor) / 2)
	and x < (window_width / 2) - ((start_button.pixel_width * global_scale_factor) / 2) + (start_button.pixel_width * global_scale_factor)
	and y > (window_height / 2) - ((start_button.pixel_height * global_scale_factor) / 2)
	and y < (window_height / 2) - ((start_button.pixel_height * global_scale_factor) / 2) + (start_button.pixel_height * global_scale_factor) then
		if player == nil then
			player_init()
		else
			player.pos_x = (window_width / 2) - ((player.pixel_width * player.scale_factor) / 2)
			player.pos_y = (window_height / 2) - ((player.pixel_height * player.scale_factor)/2)
		end
		game_paused = 0
	end
end

function is_colliding()
	if game_paused == 1 then
		return 0
	end
	
	for i, r in pairs(rockets) do
		if player.pos_x < r.pos_x + (r.pixel_width * global_scale_factor) 
		and player.pos_x + (player.pixel_width * global_scale_factor)> r.pos_x
		and player.pos_y < r.pos_y + (r.pixel_height * global_scale_factor) 
		and player.pos_y + (player.pixel_height * global_scale_factor) > r.pos_y then
			return 1
		end
	end
	return 0
end

function player_movement(dt)
	player.velocity_x = input_dir.x
	player.velocity_y = input_dir.y

	player.pos_x = player.pos_x + (player.velocity_x * player.speed) * dt
	player.pos_y = player.pos_y + (player.velocity_y * player.speed) * dt

	if player.pos_x <= 0 then 
		player.pos_x = 0
	elseif player.pos_x >= window_width - (player.pixel_width * player.scale_factor) then
		player.pos_x = window_width - (player.pixel_width * player.scale_factor)
	end

	if player.pos_y <= 0 then
		player.pos_y = 0
	elseif player.pos_y >= (window_height - (hud.pixel_height * global_scale_factor)) - (player.pixel_height * player.scale_factor) then
		player.pos_y = (window_height - (hud.pixel_height * global_scale_factor)) - (player.pixel_height * player.scale_factor)
	end

	player.rect_x = player.pos_x + (player.pixel_width * player.scale_factor)
	player.rect_y = player.pos_y + (player.pixel_height * player.scale_factor)
end

function rocket_movement(dt)
	for i, r in pairs(rockets) do 
		r.pos_x = r.pos_x + (r.speed * r.dir) * dt
		if r.pos_x >= 0 and r.pos_x + (r.pixel_width * global_scale_factor) <= window_width then
			r.visible = 1
		end
		if (r.pos_x < 0 - (r.pixel_width * global_scale_factor) or r.pos_x >= window_width) and r.visible == 1 then
			rockets[i] = nil
		end
	end
end

function love.update(dt)
	if game_paused == 1 then
		rockets = {}
		return
	end
	player_movement(dt)
	if (love.timer.getTime()) - last_rocket_spawn >= rocket_cooldown then
		last_rocket_spawn = love.timer.getTime()

		local rng = math.random(2)
		rocket_init(rng)
	end
	rocket_movement(dt)

	game_paused = is_colliding()
end

function love.draw()
	if game_paused == 1 then
		love.graphics.clear(0.21, 0.36, 0, 1)
		love.graphics.draw(
			start_button.sprite, 
			(window_width / 2) - ((start_button.pixel_width * global_scale_factor) / 2),
			(window_height / 2) - ((start_button.pixel_height * global_scale_factor) / 2),
			0,
			global_scale_factor,
			global_scale_factor
		)
		return
	end

	--clear canvas
	love.graphics.draw(background.sprite, 0, 0, 0, global_scale_factor, global_scale_factor)

	-- draw obstacles
	for i, j in pairs(obstacles) do
		love.graphics.draw(j.sprite, j.pos_x, j.pos_y, 0, j.scale_factor, j.scale_factor) 
	end

	--draw player
	love.graphics.draw(
		player.sprite, 
		player.pos_x, 
		player.pos_y, 
		player.rotation, 
		player.scale_factor, 
		player.scale_factor
	)
	for i, r in pairs(rockets) do
		love.graphics.draw(r.sprite, r.pos_x, r.pos_y, r.rotation, r.scale_factor_x, r.scale_factor_y)
	end

	love.graphics.draw(hud.sprite, 0, window_height - (hud.pixel_height * global_scale_factor), 0, global_scale_factor, global_scale_factor)

end
